#include "network.h"
#include <qsocketnotifier.h>

#include <kdebug.h>

#ifdef WITH_GGZ
Network *Network::self;
#endif

Network::Network()
: QObject()
{
#ifdef WITH_GGZ
	self = this;
	mod = NULL;
#endif
}

Network::~Network()
{
}

void Network::connect()
{
#ifdef WITH_GGZ
	int fd;
	QSocketNotifier *sn;

	kdDebug() << "network::connect" << endl;

	mod = ggzmod_new(GGZMOD_GAME);
	ggzmod_set_handler(mod, GGZMOD_EVENT_SERVER, &server_input);
	//ggzmod_set_handler(mod, GGZMOD_EVENT_PLAYER, &server_input); // GGZ 0.0.7
	ggzmod_connect(mod);
	fd = ggzmod_get_fd(mod);

	sn = new QSocketNotifier(fd, QSocketNotifier::Read, this);
	QObject::connect(sn, SIGNAL(activated(int)), SLOT(slotDispatch()));
#endif
}

void Network::disconnect()
{
#ifdef WITH_GGZ
	ggzmod_disconnect(mod);
	ggzmod_free(mod);
#endif
}

void Network::slotInput(int fd)
{
	emit signalInput(fd);
}

#ifdef WITH_GGZ
void Network::server_input(GGZMod *mod, GGZModEvent e, void *data)
{
	kdDebug() << "network::server_input" << endl;
	//switch(e)
	//{
		//case GGZMOD_EVENT_SERVER:
		if(e == GGZMOD_EVENT_SERVER)
		{
			ggzmod_set_state(mod, GGZMOD_STATE_PLAYING);
			self->serverInput(*(int*)data);
		}
		//	break;
		// GGZ 0.0.7
		//case GGZMOD_EVENT_PLAYER:
		//	kdDebug() << "player event!" << endl;
		//	kdDebug() << "is spectator? " << ((int*)data)[0] << endl;
		//	kdDebug() << "num " << ((int*)data)[1] << endl;
		//	break;
	//}
}
#endif

void Network::serverInput(int fd)
{
	QSocketNotifier *sn;

	m_channel = fd;

	sn = new QSocketNotifier(fd, QSocketNotifier::Read, this);
	QObject::connect(sn, SIGNAL(activated(int)), SLOT(slotInput(int)));
}

void Network::slotDispatch()
{
#ifdef WITH_GGZ
	kdDebug() << "network::slotdispatch" << endl;
	ggzmod_dispatch(mod);
#endif
}

int Network::channel()
{
	return m_channel;
}

long Network::generator()
{
	static long gen = 0;

	return ++gen;
}

